package com.ruoyi.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.utils.SnowflakeUtil;
import com.ruoyi.domain.*;
import com.ruoyi.domain.vo.InspectionContentVO;
import com.ruoyi.domain.vo.OptionalUseCasesVO;
import com.ruoyi.mapper.ReviewDetailsMapper;
import com.ruoyi.mapper.ReviewKeyPointMapper;
import com.ruoyi.mapper.ReviewStandardMapper;
import com.ruoyi.mapper.TestUseCaseMapper;
import com.ruoyi.service.ReviewDetailsService;
import com.ruoyi.service.ReviewKeyPointService;
import com.ruoyi.service.ReviewSceneService;
import com.ruoyi.service.ReviewStandardService;
import com.ruoyi.web.request.*;
import com.ruoyi.web.response.*;
import okhttp3.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.IOException;
import java.util.*;
import java.util.stream.Collectors;

@Transactional
@Service
public class ReviewStandardServiceImpl extends ServiceImpl<ReviewStandardMapper, ReviewStandard> implements ReviewStandardService {

    @Autowired
    private ReviewKeyPointService reviewKeyPointService;

    @Autowired
    private ReviewDetailsService reviewDetailsService;

    @Autowired
    private ReviewSceneService reviewSceneService;

    @Autowired
    private ReviewStandardMapper reviewStandardMapper;

    @Autowired
    private ReviewKeyPointMapper reviewKeyPointMapper;

    @Autowired
    private ReviewDetailsMapper reviewDetailsMapper;

    @Autowired
    private TestUseCaseMapper testUseCaseMapper;

    /**
     * 查询所有审查 标准、要点、细则、场景 并组织成树状结构返回
     * @param standardId 关联到标准库中的id
     * @param type 问卷类型[system(体系审查),car(车型审查)]
     * @return
     */
    @Override
    public List<ReviewStandard> findTree(Long standardId, String type) {

        List<ReviewStandard> reviewStandardList = reviewStandardMapper.findListByStandardIdAndType(standardId, type); // 查询审查标准
        List<ReviewKeyPoint> reviewKeyPointList = reviewKeyPointService.findByStandardList(reviewStandardList); // 查询审查要点
        List<ReviewDetails> reivewDetailsList = reviewDetailsService.findByKeyPointsList(reviewKeyPointList); // 查询审查细则
        List<ReviewScene> reviewSceneList = reviewSceneService.findByDetailsList(reivewDetailsList); // 查询审查场景

        return buildTree(reviewStandardList, reviewKeyPointList, reivewDetailsList, reviewSceneList);

    }

    /**
     * 分页查询标准列表
     *
     * @return
     */
    @Override
    public List<ReviewStandardResponse> listByPage(ReviewStandardListByPageRequest request) {
        List<ReviewStandardResponse> list = reviewStandardMapper.selectReviewStandardList(request);
        return list;
    }

    /**
     * 通过标准ID查询检验内容(树状结构)
     *
     * @param id
     * @return
     */
    @Override
    public ReviewStandardInspectionContentResponse selectInspectionContent(Long id) {
        // 模拟一个审查标准集合
        ReviewStandard reviewStandard = ReviewStandard.builder().id(id).build();
        List<ReviewStandard> reviewStandardList = new ArrayList<>();
        reviewStandardList.add(reviewStandard);

        // 查询审查要点
        List<ReviewKeyPoint> reviewKeyPointList = reviewKeyPointService.findByStandardList(reviewStandardList);
        if (reviewKeyPointList.size() == 0){
            return ReviewStandardInspectionContentResponse.builder().keyPointList(new ArrayList<>()).build();
        }
        // 查询审查细则
        List<ReviewDetails> reivewDetailsList = reviewDetailsService.findByKeyPointsList(reviewKeyPointList);
        if (reivewDetailsList.size() == 0){
            List<ReviewKeyPoint> keyPoints = buildKeyPoint(reviewKeyPointList, new ArrayList<>());
            return ReviewStandardInspectionContentResponse.builder().keyPointList(keyPoints).build();
        }
        // 查询审查场景
        List<ReviewScene> reviewSceneList = reviewSceneService.findByDetailsList(reivewDetailsList);
        if (reviewSceneList.size() == 0){
            List<ReviewDetails> reviewDetails = buildDetails(reivewDetailsList, reviewSceneList);
            List<ReviewKeyPoint> keyPoints = buildKeyPoint(reviewKeyPointList, reviewDetails);
            ReviewStandardInspectionContentResponse response = ReviewStandardInspectionContentResponse.builder().keyPointList(keyPoints).build();
            return response;
        }
        // 调用方法把数据变为树状结构
        List<ReviewDetails> reviewDetails = buildDetails(reivewDetailsList, reviewSceneList);
        List<ReviewKeyPoint> keyPoints = buildKeyPoint(reviewKeyPointList, reviewDetails);
        ReviewStandardInspectionContentResponse response = ReviewStandardInspectionContentResponse.builder().keyPointList(keyPoints).build();
        return response;
    }

    /**
     * 根据标准主键ID查询车型试验的检验内容
     * @param id
     * @return
     */
    @Override
    public List<ReviewStandardTestInspectionContentResponse> selectTestInspectionContent(Long id) {
        List<ReviewStandardTestInspectionContentResponse> list = reviewStandardMapper.selectTestInspectionContent(id);
        return list;
    }

    @Override
    public List<ReviewStandardListResponse> findReviewStandardList(ReviewStandardListRequest request) {
        return reviewStandardMapper.findReviewStandardList(request);
    }

    // 标准-要点
    @Override
    public List<ReviewStandard> findKeyPointList(ReviewFindListRequest request) {

        List<ReviewKeyPoint> keyPointList = reviewKeyPointMapper.findKeyPointList(request);

        List<ReviewStandard> standardList = reviewStandardMapper.findByKeyPointList(keyPointList,request);

        return onlyBuildStandard(standardList,keyPointList);
    }

    // 标准-要点-细则
    @Override
    public List<ReviewStandard> findDetailsList(ReviewFindListRequest request) {

        List<ReviewDetails> detailsList = reviewDetailsMapper.findDetailsList(request);

        List<ReviewKeyPoint> keyPointList = reviewKeyPointMapper.findByDetailsList(detailsList);

        List<ReviewStandard> standardList = reviewStandardMapper.findByKeyPointList(keyPointList,request);

        List<ReviewKeyPoint> keyPointListTree = buildKeyPoint(keyPointList,detailsList);

        return buildStandard(standardList,keyPointListTree);
    }

    /**
     * 选择 标准-用例
     * @param standardId
     * @return
     */
    @Override
    public ReviewStandardStandardUsecaseResponse selectStandardUsecase(Long standardId) {
        ReviewStandardStandardUsecaseResponse response = new ReviewStandardStandardUsecaseResponse();
        // 查询当前所有的测试用例
        List<OptionalUseCasesVO> testUseCases = testUseCaseMapper.selectOptionalUsecase();
        // 查看此标准下所有的测试用例
        List<TestUseCase> usecaseRelationList = testUseCaseMapper.selectList(new QueryWrapper<TestUseCase>().eq("review_standard_id", standardId));
        // 此标准还可以选择的用例
//        List<OptionalUseCasesVO> optionalUseCasesVOList = testUseCases.stream().filter(a -> !a.getStandardId().equals(standardId)).collect(Collectors.toList());
        testUseCases.stream().forEach(obj -> obj.setDisabled(false));
        response.setOptionalUseCasesList(testUseCases);
        response.setSelectedUseCaseList(usecaseRelationList.stream().map(TestUseCase::getId).collect(Collectors.toList()));
        return response;
    }

    /**
     * 保存 标准-用例
     * @param request
     */
    @Override
    public void saveStandardUsecase(ReviewStandardSaveStandardUsecaseRequest request) {
        // 删除之前绑定关系
        testUseCaseMapper.update(
                null,
                new UpdateWrapper<TestUseCase>()
                        .set("review_standard_id", null)
                        .in("review_standard_id", request.getStandardId()));
        if (CollUtil.isNotEmpty(request.getUsecaseIdList())) {
            // 绑定现在的
            testUseCaseMapper.update(
                    TestUseCase.builder().reviewStandardId(request.getStandardId()).build(),
                    new UpdateWrapper<TestUseCase>().in("id", request.getUsecaseIdList()));
        }
    }

    /**
     * 检验内容库科恩获取车型试验
     * @param request
     * @return
     * @throws IOException
     */
    @Override
    public InspectionContentResponse getList(String request) throws IOException {
        InspectionContentResponse inspectionContentResponse = new InspectionContentResponse();

        OkHttpClient client = new OkHttpClient().newBuilder().build();

        MediaType mediaType = MediaType.parse("application/json");

        // 创建请求体
        RequestBody body = RequestBody.create(mediaType, "{\"query\":\"query regulationItems($name:String!$offset:OffsetConnectionInput$orderBy:OrderByInput$searchFields:Map){regulationByName(name:$name){items(offset:$offset orderBy:$orderBy searchFields:$searchFields){totalCount nodes{id serialNumber requirements testMethod}}}}\",\"variables\":" + request+ "}");

        Request requestKE = new Request.Builder()
                .url("http://10.12.48.80:8089/api/query")
                .method("POST", body)
                .addHeader("Content-Type", "application/json")
                // TODO 连接
                .addHeader("Authorization", "Bearer eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJleHAiOjAsInRva2VuaWQiOjYxLCJ0eXBlIjoiYXBpIiwidXNlcm5hbWUiOiJhZG1pbiJ9.48Ggjx-FtOcecf73vAHn0XglwgLXZlfXxhXiLDHWWQE")
//                .addHeader("Host", "10.12.48.80:8089")
//                .addHeader("Origin", "http://10.12.48.80:8089")
//                .addHeader("Referer", "http://10.12.48.80:8089/api/graphql/playground")
//                .addHeader("User-Agent", "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/123.0.0.0 Safari/537.36")
                .build();

        Response response = client.newCall(requestKE).execute();
        String string = response.body().string();
        JSONObject jsonObject = JSONObject.parseObject(string);

        if (jsonObject.getJSONObject("data").getJSONObject("regulationByName") != null) {
            // 获取测试用例
            List<InspectionContentVO> list = jsonObject.getJSONObject("data").getJSONObject("regulationByName").getJSONObject("items").getList("nodes", InspectionContentVO.class);
            inspectionContentResponse.setInspectionContent(list);
            // 获取总条数
            Integer totalCount = (Integer) jsonObject.getJSONObject("data").getJSONObject("regulationByName").getJSONObject("items").get("totalCount");
            inspectionContentResponse.setTotalCount(totalCount);

        } else {
            List<InspectionContentVO> list = new ArrayList<>();
            inspectionContentResponse.setInspectionContent(list);
            inspectionContentResponse.setTotalCount(0);
        }

        return inspectionContentResponse;

    }

    @Override
    public List<UseCaseListResponse> getUseCaseList(String request) throws IOException {
        // TODO 检验内容库查询标准下测试用例(科恩)
        return null;
    }

    /**
     * 将审查 标准、要点、细则、场景 按照父子关系拆分写入到各自的表中
     * @param reviewStandardList
     */
    @Override
    public void initData(List<ReviewStandard> reviewStandardList) {

        Date now = new Date();

        for(ReviewStandard reviewStandard : reviewStandardList) {
            initStandard(SnowflakeUtil.generate(), reviewStandard);
        }
    }

    /**
     * 保存审查标准数据
     * @param id
     * @param reviewStandard
     */
    private void initStandard(Long id, ReviewStandard reviewStandard) {
        reviewStandard.setId(id);
        initKeyPoint(id, reviewStandard.getKeyPointList());
        save(reviewStandard);
    }

    /**
     * 保存审查要点数据
     * @param parentId
     * @param reviewKeyPointList
     */
    private void initKeyPoint(Long parentId, List<ReviewKeyPoint> reviewKeyPointList) {
        for(ReviewKeyPoint reviewKeyPoint : reviewKeyPointList) {
            Long id = SnowflakeUtil.generate();
            reviewKeyPoint.setId(id);
            reviewKeyPoint.setReviewStandardId(parentId);
            initDetails(id, reviewKeyPoint.getReviewDetailsList());
            reviewKeyPointService.save(reviewKeyPoint);
        }
    }

    /**
     * 保存审查细则数据
     * @param parentId
     * @param reviewDetailsList
     */
    private void initDetails(Long parentId, List<ReviewDetails> reviewDetailsList) {
        for(ReviewDetails reviewDetails : reviewDetailsList) {
            Long id = SnowflakeUtil.generate();
            reviewDetails.setId(id);
            reviewDetails.setReviewKeypointId(parentId);
            initScene(id, reviewDetails.getReviewSceneList());
            reviewDetailsService.save(reviewDetails);
        }
    }

    /**
     * 保存审查场景数据
     * @param parentId
     * @param reviewSceneList
     */
    private void initScene(Long parentId, List<ReviewScene> reviewSceneList) {

        Date now = new Date();

        for(ReviewScene reviewScene : reviewSceneList) {
            Long id = SnowflakeUtil.generate();
            reviewScene.setId(id);
            reviewScene.setReviewDetailsId(parentId);
            reviewSceneService.save(reviewScene);

        }
    }

    /**
     * 将审查 标准、要点、细则、场景 组织成树状结构返回
     * @param reviewStandardList 审查标准集合
     * @param reviewKeyPointList 审查要点集合
     * @param reviewDetailsList  审查细则集合
     * @param reviewSceneList    审查场景集合
     * @return
     */
    private List<ReviewStandard> buildTree(List<ReviewStandard> reviewStandardList, List<ReviewKeyPoint> reviewKeyPointList, List<ReviewDetails> reviewDetailsList, List<ReviewScene> reviewSceneList) {

        List<ReviewDetails> reviewDetails = buildDetails(reviewDetailsList, reviewSceneList);
        List<ReviewKeyPoint> keyPoints = buildKeyPoint(reviewKeyPointList, reviewDetails);
        List<ReviewStandard> standards = buildStandard(reviewStandardList, keyPoints);

        return standards;
    }

    /**
     * 构建父子关系 - 审查细则&审查场景
     * @param reviewDetailsList 审查细则集合
     * @param reviewSceneList   审查场景集合
     * @return
     */
    private List<ReviewDetails> buildDetails(List<ReviewDetails> reviewDetailsList, List<ReviewScene> reviewSceneList) {

        for(ReviewDetails reviewDetails : reviewDetailsList) {

            Long detailsId = reviewDetails.getId();
            List<ReviewScene> children = new ArrayList<>();

            for(ReviewScene reviewScene : reviewSceneList) {
                if(reviewScene.getReviewDetailsId().equals(detailsId)) {
                    children.add(reviewScene);
                }
            }

            if(children.size() > 0) {
                reviewDetails.setReviewSceneList(children);
            }

        }

        return reviewDetailsList;

    }

    /**
     * 构建父子关系 - 审查要点&审查细则
     * @param reviewKeyPointList 审查要点集合
     * @param reviewDetailsList  审查细则集合
     * @return
     */
    private List<ReviewKeyPoint> buildKeyPoint(List<ReviewKeyPoint> reviewKeyPointList, List<ReviewDetails> reviewDetailsList) {

        for(ReviewKeyPoint reviewKeyPoint : reviewKeyPointList) {

            Long keyPointId = reviewKeyPoint.getId();
            List<ReviewDetails> children = new ArrayList<>();

            for(ReviewDetails reviewDetails : reviewDetailsList) {
                if(reviewDetails.getReviewKeypointId().equals(keyPointId)) {
                    reviewDetails.setReviewKeypointText(reviewKeyPoint.getText());
                    children.add(reviewDetails);
                }
            }

            if(children.size() > 0) {
                reviewKeyPoint.setReviewDetailsList(children);
            }
        }


        return reviewKeyPointList;
    }

    /**
     * 构建负责关系 - 审查标准&审查要点
     * @param reviewStandardList 审查标准集合
     * @param reviewKeyPointList 审查要点集合
     * @return
     */
    private List<ReviewStandard> buildStandard(List<ReviewStandard> reviewStandardList, List<ReviewKeyPoint> reviewKeyPointList) {

        for(ReviewStandard reviewStandard : reviewStandardList) {

            Long standardId = reviewStandard.getId();
            List<ReviewKeyPoint> children = new ArrayList<>();

            for(ReviewKeyPoint reviewKeyPoint : reviewKeyPointList) {
                if(reviewKeyPoint.getReviewStandardId().equals(standardId)) {

                    // 为该要点下的所有细则冗余标准信息，便于前端显示
                    reviewKeyPoint.getReviewDetailsList().stream().forEach(details -> {
                        details.setStandardText(reviewStandard.getText());
                        details.setStandardId(reviewStandard.getId());
                        details.setChapter(reviewStandard.getChapter());
                    });

                    children.add(reviewKeyPoint);
                }
            }

            if(children.size() > 0) {
                reviewStandard.setKeyPointList(children);
            }

        }

        return reviewStandardList;
    }

    /**
     * 构建负责关系 - 只构建 审查标准&审查要点
     * @param reviewStandardList
     * @param reviewKeyPointList
     * @return
     */
    private List<ReviewStandard> onlyBuildStandard(List<ReviewStandard> reviewStandardList, List<ReviewKeyPoint> reviewKeyPointList) {

        for(ReviewStandard reviewStandard : reviewStandardList) {

            Long standardId = reviewStandard.getId();
            List<ReviewKeyPoint> children = new ArrayList<>();

            for(ReviewKeyPoint reviewKeyPoint : reviewKeyPointList) {
                if(reviewKeyPoint.getReviewStandardId().equals(standardId)) {
                    children.add(reviewKeyPoint);
                }
            }

            if(children.size() > 0) {
                reviewStandard.setKeyPointList(children);
            }

        }

        return reviewStandardList;
    }


    /*
     * 标准内容库新界面接口
     * */
    @Override
    public ReviewStandardListNewResponse getListNew(StandardListNewRequest request) {
        // 获取标准类型列表
        List<StandardTypeListResponse> typeListResponses = reviewStandardMapper.findTypeByStandard(request);

        // 获取章节列表
        List<ReviewStandardChapterListResponse> chapterList = reviewStandardMapper.findChaptersByStandard(request);

        // 去重类型列表，并为每种类型分配章节列表
        Set<String> uniqueTypes = new HashSet<>();
        List<StandardTypeListResponse> uniqueTypeList = typeListResponses.stream()
                .filter(type -> uniqueTypes.add(type.getType()))  // 去重 type
                .peek(type -> {
                    // 过滤并设置对应的章节列表
                    List<ReviewStandardChapterListResponse> filteredChapters = chapterList.stream()
                            .filter(chapter -> chapter.getType().equals(type.getType()))  // 这里使用 getType 方法
                            .collect(Collectors.toList());
                    type.setReviewStandardChapterListResponses(filteredChapters);
                })
                .collect(Collectors.toList());

        // 构建响应对象
        ReviewStandardListNewResponse response = new ReviewStandardListNewResponse();
        response.setTypeList(uniqueTypeList);

        return response;
    }

    /*
     * 查询当前标准下的所有id集合
     * */
    @Override
    public List<Long> getIdList(GetUseCaseRequestNew getUseCaseRequestNew) {
        return reviewStandardMapper.getIdList(getUseCaseRequestNew);
    }

    /*
     * 查询当前标准下的所有要点id集合
     * */
    @Override
    public List<Long> getKeypointIdList(GetKeypointListRequest getKeypointListRequest) {
        return reviewStandardMapper.getKeypointIdList(getKeypointListRequest);
    }

    @Override
    public List<AddDetailsResponse> getKeypointListByIds(List<Long> idList) {
        //获取所有要点
        List<KeypointList> keypointLists = reviewStandardMapper.getKeypointListByIdList(idList);

        //按reviewStandard 分组
        Map<Long,List<KeypointList>> groupKeypoint = keypointLists.stream()
                .collect(Collectors.groupingBy(KeypointList::getReviewStandardId));
        //创建返回结果列表
        List<AddDetailsResponse> addDetailsResponseList = new ArrayList<>();
        for (Map.Entry<Long,List<KeypointList>> entry : groupKeypoint.entrySet()) {
            //获取标准信息
            ReviewStandardResponseNew reviewStandardResponseNew = reviewStandardMapper.selectReviewStandardById(entry.getKey());

            //处理标准信息为null的情况
            if(reviewStandardResponseNew == null){
                continue;
            }

            //组装
            AddDetailsResponse addDetailsResponse = new AddDetailsResponse();
            addDetailsResponse.setChapter(reviewStandardResponseNew.getChapter());
            addDetailsResponse.setId(reviewStandardResponseNew.getId());
            addDetailsResponse.setText(reviewStandardResponseNew.getText());
            addDetailsResponse.setStandardId(reviewStandardResponseNew.getStandardId());
            addDetailsResponse.setStandardNo(reviewStandardResponseNew.getStandardNo());
            addDetailsResponse.setKeypointList(entry.getValue());


            //添加到返回列表
            addDetailsResponseList.add(addDetailsResponse);
        }
        return addDetailsResponseList;
    }

    @Override
    public List<Long> getDetailsIdList(DeleteDetailsRequest deleteDetailsRequest) {
        return reviewStandardMapper.getDetailsIdList(deleteDetailsRequest);
    }
}
